@@ -35,6 +35,10 @@ impl<A: Format, B: Format> Format for (A, B) {
     }
 }
 
+pub trait FormatWithStyle<Style>: Format {
+    fn format_with_style(&self, fmt: &mut Formatter, style: &Style) -> Result<()>;
+}
+
 #[derive(Debug, thiserror::Error)]
 pub enum Error {
     #[error("max columns exceeded")]
@@ -83,12 +87,6 @@ impl<'a> RegionOptions<'a> {
         self
     }
 
-    // TODO: delete
-    pub fn trailing_columns(mut self, n: usize) -> Self {
-        self.config.max_columns = self.config.max_columns.saturating_sub(n);
-        self
-    }
-
     pub fn reset_trailing_columns(mut self, n: usize) -> Self {
         self.config.trailing_columns = n;
         self
@@ -101,7 +99,8 @@ impl<'a> RegionOptions<'a> {
         self
     }
 
-    pub fn trailing_columns2(mut self, n: usize) -> Self {
+    // TODO: s/.../add_.../
+    pub fn trailing_columns(mut self, n: usize) -> Self {
         self.config.trailing_columns += n;
         self
     }
@@ -128,20 +127,6 @@ impl<'a> RegionOptions<'a> {
             Ok(())
         })
     }
-
-    pub fn enter_with_newline<F>(self, f: F) -> Result<()>
-    where
-        F: FnOnce(&mut Formatter) -> Result<()>,
-    {
-        self.fmt.with_subregion(self.config, |fmt| {
-            fmt.write_newline()?;
-            f(fmt)?;
-            if self.check_trailing_columns {
-                fmt.check_trailing_columns()?;
-            }
-            Ok(())
-        })
-    }
 }
 
 #[derive(Debug, Clone)]
