@@ -6,6 +6,7 @@ use crate::span::{Position, Span};
 pub struct RegionConfig {
     pub indent: usize,
     pub max_columns: usize,
+    pub trailing_columns: usize,
     pub allow_multi_line: bool,
     pub allow_too_long_line: bool,
 }
@@ -15,6 +16,7 @@ impl RegionConfig {
         Self {
             indent: 0,
             max_columns,
+            trailing_columns: 0,
             allow_multi_line: true,
             allow_too_long_line: true,
         }
@@ -26,6 +28,8 @@ struct RegionState {
     next_position: Position,
     current_column: usize,
     formatted_text: String,
+    popped_parent_chars: usize,
+    skip_whitespace_until: Option<Position>,
 }
 
 #[derive(Debug)]
@@ -43,6 +47,8 @@ impl RegionWriter {
                 next_position: Position::new(0, 0, 0),
                 current_column: 0,
                 formatted_text: String::new(),
+                popped_parent_chars: 0,
+                skip_whitespace_until: None,
             },
             parent: None,
         }
@@ -60,6 +66,8 @@ impl RegionWriter {
             next_position: self.state.next_position,
             current_column: self.state.current_column,
             formatted_text: String::new(),
+            popped_parent_chars: 0,
+            skip_whitespace_until: self.state.skip_whitespace_until,
         };
         let parent = std::mem::replace(
             self,
@@ -77,6 +85,10 @@ impl RegionWriter {
         let commited = std::mem::replace(self, parent);
         self.state.next_position = commited.state.next_position;
         self.state.current_column = commited.state.current_column;
+        self.state.skip_whitespace_until = commited.state.skip_whitespace_until;
+        for _ in 0..commited.state.popped_parent_chars {
+            self.pop_last_char();
+        }
         self.state
             .formatted_text
             .push_str(&commited.state.formatted_text);
@@ -87,10 +99,28 @@ impl RegionWriter {
         let _ = std::mem::replace(self, parent);
     }
 
+    pub fn skip_whitespace_until(&mut self, position: Position) {
+        assert!(self.state.skip_whitespace_until.is_none());
+        self.state.skip_whitespace_until = Some(position);
+    }
+
     pub fn config(&self) -> &RegionConfig {
         &self.config
     }
 
+    pub fn parent_indent(&self) -> usize {
+        let current = self.config.indent;
+        if let Some(parent) = &self.parent {
+            if parent.config().indent != current {
+                parent.config().indent
+            } else {
+                parent.parent_indent()
+            }
+        } else {
+            current
+        }
+    }
+
     pub fn next_position(&self) -> Position {
         self.state.next_position
     }
@@ -104,6 +134,10 @@ impl RegionWriter {
     }
 
     pub fn write_space(&mut self) -> Result<()> {
+        if self.state.skip_whitespace_until.is_some() {
+            return Ok(());
+        }
+
         if !matches!(self.last_char(), ' ' | '\n') {
             self.write(" ")?;
         }
@@ -111,8 +145,12 @@ impl RegionWriter {
     }
 
     pub fn write_newline(&mut self) -> Result<()> {
+        if self.state.skip_whitespace_until.is_some() {
+            return Ok(());
+        }
+
         if self.last_char() != '\n' {
-            if self.last_char() == ' ' {
+            if self.last_char() == ' ' && self.next_last_char() != '$' {
                 self.pop_last_char();
             }
             self.write("\n")?;
@@ -121,16 +159,33 @@ impl RegionWriter {
     }
 
     pub fn write_item(&mut self, text: &str, item: &impl Span) -> Result<()> {
+        if self
+            .state
+            .skip_whitespace_until
+            .map_or(false, |p| p < item.end_position())
+        {
+            self.state.skip_whitespace_until = None;
+        }
+
         // If macros have appeared in the text,
         // an item's start and end positions could be smaller than `next_position`.
         let start = std::cmp::max(item.start_position(), self.state.next_position);
         let end = std::cmp::max(item.end_position(), start);
-        let text = text[start.offset()..end.offset()].trim();
+
+        // `trim_start()` is sometimes needed for macros such as `?a(?b c)`.
+        let text = text[start.offset()..end.offset()].trim_start();
 
         if self.state.next_position.line() + 1 < item.start_position().line() {
             self.write("\n")?;
         }
 
+        if !text.is_empty() && self.last_char() == '\n' {
+            for _ in 0..self.config.indent {
+                self.state.formatted_text.push(' ');
+            }
+            self.state.current_column = self.config.indent;
+        }
+
         self.write(text)?;
         self.state.next_position = end;
         Ok(())
@@ -139,11 +194,21 @@ impl RegionWriter {
     pub fn write_comment(&mut self, text: &str, comment: &CommentToken) -> Result<()> {
         assert!(!comment.is_empty());
 
+        if self
+            .state
+            .skip_whitespace_until
+            .map_or(false, |p| p < comment.end_position())
+        {
+            self.state.skip_whitespace_until = None;
+        }
+
         match comment.kind() {
             CommentKind::Post => {
-                if self.last_char() != '\n' {
-                    self.write("\n")?;
-                }
+                self.write_newline()?;
+                // if self.last_char() != '\n' {
+                //     // self.write("\n")?;
+
+                // }
 
                 if self.state.next_position.line() + 1 < comment.start_position().line() {
                     self.write("\n")?;
@@ -179,19 +244,11 @@ impl RegionWriter {
                     return Err(Error::MultiLine);
                 }
                 self.state.current_column = 0;
-            } else if c != ' ' {
-                if self.state.current_column >= self.config.max_columns
-                    && !self.config.allow_too_long_line
-                {
-                    return Err(Error::LineTooLong);
-                }
-
-                if self.state.current_column < self.config.indent {
-                    for _ in self.state.current_column..self.config.indent {
-                        self.state.formatted_text.push(' ');
-                    }
-                    self.state.current_column = self.config.indent;
-                }
+            } else if (self.last_char() == '$' || c != ' ')
+                && self.state.current_column >= self.config.max_columns
+                && !self.config.allow_too_long_line
+            {
+                return Err(Error::LineTooLong);
             }
 
             self.state.formatted_text.push(c);
@@ -203,10 +260,19 @@ impl RegionWriter {
     }
 
     pub fn last_char(&self) -> char {
-        if let Some(c) = self.state.formatted_text.chars().last() {
+        self.n_last_char(0)
+    }
+
+    fn next_last_char(&self) -> char {
+        self.n_last_char(1)
+    }
+
+    fn n_last_char(&self, n: usize) -> char {
+        if let Some(c) = self.state.formatted_text.chars().rev().nth(n) {
             c
         } else if let Some(parent) = &self.parent {
-            parent.last_char()
+            let m = self.state.formatted_text.chars().count();
+            parent.n_last_char(self.state.popped_parent_chars + (n - m))
         } else {
             '\n'
         }
@@ -214,9 +280,7 @@ impl RegionWriter {
 
     fn pop_last_char(&mut self) {
         if self.state.formatted_text.pop().is_none() {
-            if let Some(x) = &mut self.parent {
-                x.pop_last_char();
-            }
+            self.state.popped_parent_chars += 1;
         }
     }
 }
