@@ -11,15 +11,13 @@ pub(crate) mod atoms;
 pub(crate) mod generics;
 pub(crate) mod keywords;
 pub(crate) mod macros;
-pub(crate) mod qualifiers;
-pub(crate) mod styles;
 pub(crate) mod symbols;
 pub(crate) mod variables;
 
 mod module;
 
 pub use self::macros::Macro;
-pub use self::module::{LooseModule, Module};
+pub use self::module::Module;
 
 /// One of [forms].
 #[derive(Debug, Clone, Span, Parse, Format)]
