@@ -1,8 +1,11 @@
+//! Erlang expressions.
+use self::bitstrings::BitstringExpr;
+use self::blocks::BlockExpr;
+use self::functions::FunctionExpr;
+use self::records::{RecordAccessOrUpdateExpr, RecordConstructOrIndexExpr};
 use crate::format::{self, Format};
-use crate::items::generics::{Either, NonEmptyItems, Parenthesized};
-use crate::items::keywords::WhenKeyword;
-use crate::items::styles::{Child, ColumnIndent, Newline, Space};
-use crate::items::symbols::{CommaSymbol, OpenBraceSymbol, SemicolonSymbol};
+use crate::items::generics::{BinaryOpLike, BinaryOpStyle, Either, NonEmptyItems, Parenthesized};
+use crate::items::symbols::{CommaSymbol, DoubleLeftArrowSymbol, LeftArrowSymbol, OpenBraceSymbol};
 use crate::items::tokens::{
     AtomToken, CharToken, FloatToken, IntegerToken, SymbolToken, Token, VariableToken,
 };
@@ -10,23 +13,23 @@ use crate::parse::{self, Parse};
 use crate::span::Span;
 use erl_tokenize::values::{Keyword, Symbol};
 
-pub mod bitstrings;
-pub mod blocks;
-pub mod calls;
-pub mod functions;
-pub mod lists;
-pub mod maps;
-pub mod records;
-pub mod strings;
-pub mod tuples;
+mod bitstrings;
+mod blocks;
+mod calls;
+pub(crate) mod functions; // TODO
+mod lists;
+pub(crate) mod maps;
+mod records;
+mod strings;
+mod tuples;
 
-pub use self::bitstrings::BitstringExpr;
-pub use self::blocks::BlockExpr;
-pub use self::calls::{BinaryOp, BinaryOpCallExpr, FunctionCallExpr, UnaryOpCallExpr};
-pub use self::functions::FunctionExpr;
-pub use self::lists::ListExpr;
+pub use self::bitstrings::{BitstringComprehensionExpr, BitstringConstructExpr};
+pub use self::blocks::{BeginExpr, CaseExpr, CatchExpr, IfExpr, ReceiveExpr, TryExpr};
+pub use self::calls::{BinaryOp, BinaryOpCallExpr, FunctionCallExpr, UnaryOp, UnaryOpCallExpr};
+pub use self::functions::{AnonymousFunctionExpr, DefinedFunctionExpr, NamedFunctionExpr};
+pub use self::lists::{ListComprehensionExpr, ListConstructExpr, ListExpr};
 pub use self::maps::{MapConstructExpr, MapUpdateExpr};
-pub use self::records::{RecordAccessOrUpdateExpr, RecordConstructOrIndexExpr};
+pub use self::records::{RecordAccessExpr, RecordConstructExpr, RecordIndexExpr, RecordUpdateExpr};
 pub use self::strings::StringExpr;
 pub use self::tuples::TupleExpr;
 
@@ -154,6 +157,7 @@ impl Expr {
     }
 }
 
+/// [AtomToken] | [CharToken] | [FloatToken] | [IntegerToken] | [VariableToken] | [StringExpr]
 #[derive(Debug, Clone, Span, Parse, Format)]
 pub enum LiteralExpr {
     Atom(AtomToken),
@@ -164,9 +168,9 @@ pub enum LiteralExpr {
     VariableToken(VariableToken),
 }
 
-// TODO: delete?
+// TODO: s/AtomLikExpr/LiteralOrParen.../
 #[derive(Debug, Clone, Span, Parse, Format)]
-pub enum AtomLikeExpr {
+enum AtomLikeExpr {
     Atom(AtomToken),
     Variable(VariableToken),
     Expr(Parenthesized<Expr>),
@@ -183,87 +187,56 @@ impl From<AtomLikeExpr> for BaseExpr {
 }
 
 #[derive(Debug, Clone, Span, Parse, Format)]
-pub enum IntegerLikeExpr {
+enum IntegerLikeExpr {
     Integer(IntegerToken),
     Variable(VariableToken),
     Expr(Parenthesized<Expr>),
 }
 
 #[derive(Debug, Clone, Span, Parse)]
-pub struct Body {
-    exprs: NonEmptyItems<Child<Expr>, Newline<CommaSymbol>>,
+struct Body {
+    exprs: NonEmptyItems<Expr, CommaSymbol>,
+}
+
+impl Body {
+    fn exprs(&self) -> &[Expr] {
+        self.exprs.items()
+    }
 }
 
 impl Format for Body {
     fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
-        fmt.subregion()
-            .indent_offset(4)
-            .trailing_columns(1) // '.' or ',' or ';' (TODO move other place?)
-            .enter(|fmt| {
-                fmt.write_newline()?;
-                self.exprs.format(fmt)
-            })
-    }
-}
-
-#[derive(Debug, Clone, Span, Parse)]
-pub struct MaybeInlineBody {
-    exprs: NonEmptyItems<Child<Expr>, Newline<CommaSymbol>>,
-}
-
-impl Format for MaybeInlineBody {
-    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
-        let mut options = fmt.subregion().trailing_columns(4); // ' end'
-        if self.exprs.get().len() > 1 {
-            options = options.indent_offset(4)
-        }
-        options.enter(|fmt| {
-            if self.exprs.get().len() > 1 {
-                fmt.write_newline()?;
-            }
-            self.exprs.format(fmt)
+        fmt.subregion().indent_offset(4).enter(|fmt| {
+            fmt.write_newline()?;
+            self.exprs.format_multiline(fmt)
         })
     }
 }
 
-#[derive(Debug, Clone, Span, Parse)]
-pub struct Guard {
-    when: Space<WhenKeyword>,
-    condition: ColumnIndent<GuardCondition>,
-}
-
-impl Format for Guard {
-    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
-        let format = |fmt: &mut format::Formatter| {
-            self.when.format(fmt)?;
-            self.condition.format(fmt)?;
-            Ok(())
-        };
-
-        if fmt.current_relative_column() < 2 {
-            fmt.subregion()
-                .trailing_columns(3) // ' ->'
-                .enter(format)?;
-        } else if fmt
-            .subregion()
-            .trailing_columns(3) // ' ->'
-            .forbid_too_long_line()
-            .forbid_multi_line()
-            .enter(format)
-            .is_err()
-        {
-            fmt.subregion()
-                .indent_offset(2)
-                .trailing_columns(3) // ' ->'
-                .enter_with_newline(format)?;
-        }
-        Ok(())
-    }
+#[derive(Debug, Clone, Span, Parse, Format)]
+enum Qualifier {
+    Generator(Generator),
+    Filter(Expr),
 }
 
 #[derive(Debug, Clone, Span, Parse, Format)]
-pub struct GuardCondition {
-    conditions: NonEmptyItems<Expr, Either<CommaSymbol, SemicolonSymbol>>,
+struct Generator(BinaryOpLike<Expr, GeneratorDelimiter, Expr>);
+
+#[derive(Debug, Clone, Span, Parse, Format)]
+struct GeneratorDelimiter(Either<LeftArrowSymbol, DoubleLeftArrowSymbol>);
+
+impl BinaryOpStyle for GeneratorDelimiter {
+    fn indent_offset(&self) -> usize {
+        4
+    }
+
+    fn allow_newline(&self) -> bool {
+        false
+    }
+
+    fn should_pack(&self) -> bool {
+        false
+    }
 }
 
 #[cfg(test)]
@@ -277,6 +250,7 @@ mod tests {
             foo(A) when A ->
                 A."},
             indoc::indoc! {"
+            %---10---|%---20---|
             foo(A, B, C)
               when A =:= B ->
                 C."},
