@@ -1,8 +1,6 @@
 use crate::format::{self, Format};
-use crate::items::expressions::BaseExpr;
-use crate::items::generics::{Elements, Maybe, NonEmptyItems};
-use crate::items::qualifiers::Qualifier;
-use crate::items::styles::{ColumnIndent, RightSpace, Space};
+use crate::items::expressions::{BaseExpr, Expr, Qualifier};
+use crate::items::generics::{BinaryOpLike, BinaryOpStyle, BitstringLike, Maybe, NonEmptyItems};
 use crate::items::symbols::{
     ColonSymbol, DoubleLeftAngleSymbol, DoubleRightAngleSymbol, DoubleVerticalBarSymbol,
     HyphenSymbol, SlashSymbol,
@@ -17,24 +15,56 @@ pub enum BitstringExpr {
     Comprehension(BitstringComprehensionExpr),
 }
 
+/// `<<` (`$SEGMENT` `,`?)* `>>`
+///
+/// - $SEGMENT: [Expr] `$SIZE`? `$TYPE`?
+/// - $SIZE: `:` [Expr]
+/// - $TYPE: `/` ([AtomToken] `-`?)+
 #[derive(Debug, Clone, Span, Parse, Format)]
-pub struct BitstringConstructExpr {
+pub struct BitstringConstructExpr(BitstringLike<BitstringSegment>);
+
+/// `<<` [Expr] `||` ([Qualifier] `,`?)+  `>>`
+// TODO: Use `ComprehensionLike`
+#[derive(Debug, Clone, Span, Parse)]
+pub struct BitstringComprehensionExpr {
     open: DoubleLeftAngleSymbol,
-    segments: Elements<BitstringSegment>,
+    body: ComprehensionBody,
     close: DoubleRightAngleSymbol,
 }
 
+impl Format for BitstringComprehensionExpr {
+    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
+        self.open.format(fmt)?;
+        fmt.subregion()
+            .current_column_as_indent()
+            .reset_trailing_columns(2) // ">>"
+            .enter(|fmt| self.body.format(fmt))?;
+        self.close.format(fmt)?;
+        Ok(())
+    }
+}
+
+type ComprehensionBody = BinaryOpLike<Expr, ComprehensionDelimiter, NonEmptyItems<Qualifier>>;
+
 #[derive(Debug, Clone, Span, Parse, Format)]
-pub struct BitstringComprehensionExpr {
-    open: RightSpace<DoubleLeftAngleSymbol>,
-    item: BaseExpr,
-    bar: Space<DoubleVerticalBarSymbol>,
-    qualifiers: ColumnIndent<NonEmptyItems<Qualifier>>,
-    close: DoubleRightAngleSymbol,
+struct ComprehensionDelimiter(DoubleVerticalBarSymbol);
+
+impl BinaryOpStyle for ComprehensionDelimiter {
+    fn indent_offset(&self) -> usize {
+        2
+    }
+
+    fn allow_newline(&self) -> bool {
+        true
+    }
+
+    fn should_pack(&self) -> bool {
+        false
+    }
 }
 
 #[derive(Debug, Clone, Span, Parse)]
-pub struct BitstringSegment {
+struct BitstringSegment {
     value: BaseExpr,
     size: Maybe<BitstringSegmentSize>,
     ty: Maybe<BitstringSegmentType>,
@@ -54,13 +84,13 @@ impl Format for BitstringSegment {
 }
 
 #[derive(Debug, Clone, Span, Parse, Format)]
-pub struct BitstringSegmentSize {
+struct BitstringSegmentSize {
     colon: ColonSymbol,
     size: BaseExpr,
 }
 
 #[derive(Debug, Clone, Span, Parse)]
-pub struct BitstringSegmentType {
+struct BitstringSegmentType {
     slash: SlashSymbol,
     specifiers: NonEmptyItems<BitstringSegmentTypeSpecifier, HyphenSymbol>,
 }
@@ -70,15 +100,15 @@ impl Format for BitstringSegmentType {
         self.slash.format(fmt)?;
         for (item, delimiter) in self
             .specifiers
-            .items
+            .items()
             .iter()
-            .zip(self.specifiers.delimiters.iter())
+            .zip(self.specifiers.delimiters().iter())
         {
             item.format(fmt)?;
             delimiter.format(fmt)?;
         }
         self.specifiers
-            .items
+            .items()
             .last()
             .expect("unreachable")
             .format(fmt)?;
@@ -87,7 +117,7 @@ impl Format for BitstringSegmentType {
 }
 
 #[derive(Debug, Clone, Span, Parse, Format)]
-pub struct BitstringSegmentTypeSpecifier {
+struct BitstringSegmentTypeSpecifier {
     name: AtomToken,
     value: Maybe<(ColonSymbol, IntegerToken)>,
 }
@@ -101,8 +131,9 @@ mod tests {
         let texts = [
             "<<>>",
             indoc::indoc! {"
-            <<1, 2, 3, 4, 5,
-              6, 7, 8, 9>>"},
+            %---10---|%---20---|
+            <<1, 2, 3, 4, 5, 6,
+              7, 8, 9>>"},
             "<<1, 2:16, 3>>",
             "<<<<\"foo\">>/binary>>",
             indoc::indoc! {"
@@ -122,29 +153,30 @@ mod tests {
     fn bitstring_comprehension_works() {
         let texts = [
             indoc::indoc! {"
-            << <<X>> || X <- [1,
-                              2,
-                              3]>>"},
+            %---10---|%---20---|
+            <<<<X>> ||
+                X <- [1, 2, 3]>>"},
             indoc::indoc! {"
-            << (foo(X,
-                    Y,
-                    Z,
-                    bar(),
-                    baz())) || X <- [1,
-                                     2,
-                                     3],
-                               Y <= Z,
-                               false>>"},
+            %---10---|%---20---|
+            <<(foo(X,
+                   Y,
+                   Z,
+                   bar(),
+                   baz())) ||
+                X <- [1, 2, 3,
+                      4, 5],
+                Y <= Z,
+                false>>"},
             indoc::indoc! {"
-            << <<if
-                     X < 10 ->
-                         X +
-                         $0;
-                     true ->
-                         X -
-                         10 +
-                         $A
-                 end>> || <<X:4>> <= B>>"},
+            %---10---|%---20---|
+            <<<<if
+                    X < 10 ->
+                        X + $0;
+                    true ->
+                        X - 10 +
+                        $A
+                end>> ||
+                <<X:4>> <= B>>"},
         ];
         for text in texts {
             crate::assert_format!(text, Expr);
