@@ -1,12 +1,11 @@
-use crate::format::Format;
-use crate::items::expressions::{Body, Expr, Guard, GuardCondition};
-use crate::items::generics::{Clauses, Either, Maybe, NonEmptyItems};
+use crate::format::{self, Format};
+use crate::items::expressions::{Body, Expr};
+use crate::items::generics::{Clauses, Either, Maybe, NonEmptyItems, WithArrow, WithGuard};
 use crate::items::keywords::{
     AfterKeyword, BeginKeyword, CaseKeyword, CatchKeyword, EndKeyword, IfKeyword, OfKeyword,
     ReceiveKeyword, TryKeyword,
 };
-use crate::items::styles::{Block, Child, ColumnIndent, Newline, RightSpace, Space};
-use crate::items::symbols::{ColonSymbol, RightArrowSymbol};
+use crate::items::symbols::{ColonSymbol, CommaSymbol, SemicolonSymbol};
 use crate::items::tokens::{AtomToken, VariableToken};
 use crate::parse::Parse;
 use crate::span::Span;
@@ -21,101 +20,224 @@ pub enum BlockExpr {
     Catch(Box<CatchExpr>),
 }
 
-#[derive(Debug, Clone, Span, Parse, Format)]
+/// `case` [Expr] `of` (`$CLAUSE` `;`?)+ `end`
+///
+/// - $CLAUSE: `$PATTERN` (`when` `$GUARD`)? `->` `$BODY`
+/// - $PATTERN: [Expr]
+/// - $GUARD: ([Expr] (`,` | `;`)?)+
+/// - $BODY: ([Expr] `,`?)+
+#[derive(Debug, Clone, Span, Parse)]
 pub struct CaseExpr {
-    case: Space<CaseKeyword>,
-    value: Child<Expr>,
-    of: Space<OfKeyword>,
-    clauses: Newline<Block<Clauses<CaseClause>>>,
+    case: CaseKeyword,
+    value: Expr,
+    of: OfKeyword,
+    clauses: Block<Clauses<CaseClause>>,
     end: EndKeyword,
 }
 
-#[derive(Debug, Clone, Span, Parse, Format)]
-pub struct CaseClause {
-    pattern: Expr,
-    guard: Maybe<Guard>,
-    arrow: Space<RightArrowSymbol>,
-    body: Body,
+impl Format for CaseExpr {
+    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
+        fmt.subregion().clear_trailing_columns(true).enter(|fmt| {
+            self.case.format(fmt)?;
+            fmt.write_space()?;
+
+            fmt.subregion()
+                .reset_trailing_columns(3) // " of"
+                .enter(|fmt| self.value.format(fmt))?;
+            fmt.write_space()?;
+
+            self.of.format(fmt)?;
+            self.clauses.format(fmt)?;
+            self.end.format(fmt)?;
+
+            Ok(())
+        })
+    }
 }
 
 #[derive(Debug, Clone, Span, Parse, Format)]
+struct CaseClause {
+    pattern: WithArrow<WithGuard<Expr, Expr>>,
+    body: Body,
+}
+
+/// `if` (`$CLAUSE` `;`?)+ `end`
+///
+/// - $CLAUSE: `$GUARD` `->` `$BODY`
+/// - $GUARD: ([Expr] (`,` | `;`)?)+
+/// - $BODY: ([Expr] `,`?)+
+#[derive(Debug, Clone, Span, Parse)]
 pub struct IfExpr {
     r#if: IfKeyword,
-    clauses: Newline<Block<Clauses<IfClause>>>,
+    clauses: Block<Clauses<IfClause>>,
     end: EndKeyword,
 }
 
+impl Format for IfExpr {
+    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
+        fmt.subregion().clear_trailing_columns(true).enter(|fmt| {
+            self.r#if.format(fmt)?;
+            self.clauses.format(fmt)?;
+            self.end.format(fmt)?;
+            Ok(())
+        })
+    }
+}
+
 #[derive(Debug, Clone, Span, Parse, Format)]
-pub struct IfClause {
-    condigion: GuardCondition,
-    arrow: Space<RightArrowSymbol>,
+struct IfClause {
+    condigion: WithArrow<GuardCondition>,
     body: Body,
 }
 
+#[derive(Debug, Clone, Span, Parse, Format)]
+struct GuardCondition(NonEmptyItems<Expr, Either<CommaSymbol, SemicolonSymbol>>);
+
+/// `begin` `$BODY` `end`
+///
+/// - $BODY: ([Expr] `,`?)+
 #[derive(Debug, Clone, Span, Parse, Format)]
 pub struct BeginExpr {
     begin: BeginKeyword,
-    exprs: Newline<Block<NonEmptyItems<Expr>>>,
+    exprs: Block<NonEmptyItems<Expr>>,
     end: EndKeyword,
 }
 
-#[derive(Debug, Clone, Span, Parse, Format)]
+/// `receive` (`$CLAUSE` `;`?)* `$TIMEOUT`? `end`
+///
+/// - $CLAUSE: `$PATTERN` (`when` `$GUARD`)? `->` `$BODY`
+/// - $PATTERN: [Expr]
+/// - $GUARD: ([Expr] (`,` | `;`)?)+
+/// - $TIMEOUT: `after` [Expr] `->` `$BODY`
+/// - $BODY: ([Expr] `,`?)+
+#[derive(Debug, Clone, Span, Parse)]
 pub struct ReceiveExpr {
-    receive: Newline<ReceiveKeyword>,
-    clauses: Maybe<Newline<Block<Clauses<CaseClause>>>>,
-    timeout: Maybe<Newline<ReceiveTimeout>>,
+    receive: ReceiveKeyword,
+    clauses: Block<Maybe<Clauses<CaseClause>>>,
+    timeout: Maybe<ReceiveTimeout>,
     end: EndKeyword,
 }
 
+impl Format for ReceiveExpr {
+    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
+        fmt.subregion().clear_trailing_columns(true).enter(|fmt| {
+            self.receive.format(fmt)?;
+            self.clauses.format(fmt)?;
+            self.timeout.format(fmt)?;
+            self.end.format(fmt)?;
+            Ok(())
+        })
+    }
+}
+
 #[derive(Debug, Clone, Span, Parse, Format)]
-pub struct ReceiveTimeout {
-    after: Newline<AfterKeyword>,
+struct ReceiveTimeout {
+    after: AfterKeyword,
     clause: Block<ReceiveTimeoutClause>,
 }
 
 #[derive(Debug, Clone, Span, Parse, Format)]
-pub struct ReceiveTimeoutClause {
-    timeout: Expr,
-    arrow: Space<RightArrowSymbol>,
+struct ReceiveTimeoutClause {
+    timeout: WithArrow<Expr>,
     body: Body,
 }
 
-#[derive(Debug, Clone, Span, Parse, Format)]
+/// `try` `$BODY` `$BRANCHES`? `$CATCH`? `$AFTER`? `end`
+///
+/// - $BODY: ([Expr] `,`?)+
+/// - $BRANCHES: (`of` (`$BRANCH_CLAUSE` `;`?)+)
+/// - $BRANCH_CLAUSE: `$PATTERN` (`when` `$GUARD`)? `->` `$BODY`
+/// - $PATTERN: [Expr]
+/// - $GUARD: ([Expr] (`,` | `;`)?)+
+/// - $CATCH: `catch` (`$CATCH_CLAUSE` `;`?)+
+/// - $CATCH_CLAUSE: `$ERROR_CLASS`? [Expr] `$STACKTRACE`? (`when` `$GUARD`)? `->` `$BODY`
+/// - $ERROR_CLASS: ([AtomToken] | [VariableToken]) `:`
+/// - $STACKTRACE: `:` [VariableToken]
+/// - $AFTER: `after` `$BODY`
+#[derive(Debug, Clone, Span, Parse)]
 pub struct TryExpr {
     r#try: TryKeyword,
-    body: Newline<Body>,
-    clauses: Maybe<(OfKeyword, Newline<Block<Clauses<CaseClause>>>)>,
-    catch: Maybe<Newline<TryCatch>>,
-    after: Maybe<Newline<TryAfter>>,
+    body: Body,
+    clauses: Maybe<(OfKeyword, Block<Clauses<CaseClause>>)>,
+    catch: Maybe<TryCatch>,
+    after: Maybe<TryAfter>,
     end: EndKeyword,
 }
 
-#[derive(Debug, Clone, Span, Parse, Format)]
-pub struct TryCatch {
-    catch: CatchKeyword,
-    clauses: Newline<Block<Clauses<CatchClause>>>,
+impl Format for TryExpr {
+    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
+        fmt.subregion().clear_trailing_columns(true).enter(|fmt| {
+            self.r#try.format(fmt)?;
+            self.body.format(fmt)?;
+            fmt.write_newline()?;
+            self.clauses.format(fmt)?;
+            self.catch.format(fmt)?;
+            self.after.format(fmt)?;
+            fmt.write_newline()?;
+            self.end.format(fmt)?;
+            Ok(())
+        })
+    }
 }
 
 #[derive(Debug, Clone, Span, Parse, Format)]
-pub struct CatchClause {
-    class: Maybe<(Either<AtomToken, VariableToken>, ColonSymbol)>,
-    pattern: Expr,
-    stacktrace: Maybe<(ColonSymbol, VariableToken)>,
-    guard: Maybe<Guard>,
-    arrow: Space<RightArrowSymbol>,
+struct TryCatch {
+    catch: CatchKeyword,
+    clauses: Block<Clauses<CatchClause>>,
+}
+
+#[derive(Debug, Clone, Span, Parse, Format)]
+struct CatchClause {
+    pattern: WithArrow<WithGuard<CatchPattern, Expr>>,
     body: Body,
 }
 
 #[derive(Debug, Clone, Span, Parse, Format)]
-pub struct TryAfter {
+struct CatchPattern {
+    class: Maybe<(Either<AtomToken, VariableToken>, ColonSymbol)>,
+    pattern: Expr,
+    stacktrace: Maybe<(ColonSymbol, VariableToken)>,
+}
+
+#[derive(Debug, Clone, Span, Parse, Format)]
+struct TryAfter {
     after: AfterKeyword,
     body: Body,
 }
 
-#[derive(Debug, Clone, Span, Parse, Format)]
+/// `catch` [Expr]
+#[derive(Debug, Clone, Span, Parse)]
 pub struct CatchExpr {
-    catch: RightSpace<CatchKeyword>,
-    expr: ColumnIndent<Expr>,
+    catch: CatchKeyword,
+    expr: Expr,
+}
+
+impl Format for CatchExpr {
+    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
+        self.catch.format(fmt)?;
+        fmt.write_space()?;
+        fmt.subregion()
+            .current_column_as_indent()
+            .enter(|fmt| self.expr.format(fmt))?;
+        Ok(())
+    }
+}
+
+#[derive(Debug, Clone, Span, Parse)]
+struct Block<T>(T);
+
+impl<T: Format> Format for Block<T> {
+    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
+        fmt.subregion()
+            .clear_trailing_columns(true)
+            .indent_offset(4)
+            .enter(|fmt| {
+                fmt.write_newline()?;
+                self.0.format(fmt)?;
+                fmt.write_newline()?;
+                Ok(())
+            })
+    }
 }
 
 #[cfg(test)]
@@ -131,6 +253,7 @@ mod tests {
                     2
             end"},
             indoc::indoc! {"
+            %---10---|%---20---|
             case foo() of
                 {1, 2} ->
                     3;
@@ -208,14 +331,15 @@ mod tests {
     fn begin_works() {
         let texts = [
             indoc::indoc! {"
-                begin
-                    1
-                end"},
+            begin
+                1
+            end"},
             indoc::indoc! {"
-                begin
-                    foo(bar, Baz),
-                    {[#{}]}
-                end"},
+            %---10---|%---20---|
+            begin
+                foo(bar, Baz),
+                {[#{}]}
+            end"},
         ];
         for text in texts {
             crate::assert_format!(text, Expr);
@@ -282,10 +406,11 @@ mod tests {
         let texts = [
             "catch 1",
             indoc::indoc! {"
-                catch foo(bar,
-                          Baz,
-                          qux) +
-                      3 + 4"},
+            %---10---|%---20---|
+            catch foo(bar,
+                      Baz,
+                      qux) + 3 +
+                  4"},
         ];
         for text in texts {
             crate::assert_format!(text, Expr);
