@@ -1,7 +1,6 @@
 use crate::format::{self, Format};
 use crate::items::expressions::{Either, Expr};
-use crate::items::generics::Tuple;
-use crate::items::styles::Space;
+use crate::items::generics::{BinaryOpLike, BinaryOpStyle, TupleLike};
 use crate::items::symbols::{DotSymbol, MatchSymbol, SharpSymbol};
 use crate::items::tokens::AtomToken;
 use crate::items::variables::UnderscoreVariable;
@@ -30,13 +29,22 @@ impl ResumeParse<Expr> for RecordAccessOrUpdateExpr {
     }
 }
 
+/// `#` `$NAME` `{` (`$FIELD` `,`?)* `}`
+///
+/// - $NAME: [AtomToken]
+/// - $FIELD: ([AtomToken] | `_`) `=` [Expr]
 #[derive(Debug, Clone, Span, Parse, Format)]
 pub struct RecordConstructExpr {
     sharp: SharpSymbol,
     name: AtomToken,
-    fields: Tuple<RecordField, 1>,
+    fields: TupleLike<RecordField>,
 }
 
+/// `$VALUE` `#` `$NAME` `.` `$FIELD`
+///
+/// - $VALUE: [Expr]
+/// - $NAME: [AtomToken]
+/// - $FIELD: [AtomToken]
 #[derive(Debug, Clone, Span, Parse)]
 pub struct RecordAccessExpr {
     value: Expr,
@@ -63,6 +71,10 @@ impl Format for RecordAccessExpr {
     }
 }
 
+/// `#` `$NAME` `.` `$FIELD`
+///
+/// - $NAME: [AtomToken]
+/// - $FIELD: [AtomToken]
 #[derive(Debug, Clone, Span, Parse, Format)]
 pub struct RecordIndexExpr {
     sharp: SharpSymbol,
@@ -71,12 +83,17 @@ pub struct RecordIndexExpr {
     field: AtomToken,
 }
 
+/// `$VALUE` `#` `$NAME` `{` (`$FIELD` `,`?)* `}`
+///
+/// - $VALUE: [Expr]
+/// - $NAME: [AtomToken]
+/// - $FIELD: [AtomToken]
 #[derive(Debug, Clone, Span, Parse)]
 pub struct RecordUpdateExpr {
     value: Expr,
     sharp: SharpSymbol,
     name: AtomToken,
-    fields: Tuple<RecordField, 1>,
+    fields: TupleLike<RecordField>,
 }
 
 impl ResumeParse<Expr> for RecordUpdateExpr {
@@ -104,10 +121,23 @@ impl Format for RecordUpdateExpr {
 }
 
 #[derive(Debug, Clone, Span, Parse, Format)]
-pub struct RecordField {
-    name: Either<AtomToken, UnderscoreVariable>,
-    delimiter: Space<MatchSymbol>,
-    value: Expr,
+struct RecordField(BinaryOpLike<Either<AtomToken, UnderscoreVariable>, RecordFieldDelimiter, Expr>);
+
+#[derive(Debug, Clone, Span, Parse, Format)]
+struct RecordFieldDelimiter(MatchSymbol);
+
+impl BinaryOpStyle for RecordFieldDelimiter {
+    fn indent_offset(&self) -> usize {
+        4
+    }
+
+    fn allow_newline(&self) -> bool {
+        true
+    }
+
+    fn should_pack(&self) -> bool {
+        false
+    }
 }
 
 #[cfg(test)]
@@ -122,9 +152,10 @@ mod tests {
             #foo{module = Mod,
                  _ = '_'}"},
             indoc::indoc! {"
+            %---10---|%---20---|
             #foo{bar = 2,
-                 baz = {bar,
-                        baz}}"},
+                 baz =
+                     {bar, baz}}"},
         ];
         for text in texts {
             crate::assert_format!(text, Expr);
@@ -157,12 +188,17 @@ mod tests {
         let texts = [
             "M#foo{}",
             "88 #foo{}",
-            "M#foo.bar#baz{qux = 1}",
             "M#baz{qux = 1}#foo.bar",
             indoc::indoc! {"
-                (foo())#foo{bar = 2,
-                            baz = {bar,
-                                   baz}}"},
+            %---10---|%---20---|
+            M#foo.bar#baz{qux =
+                              1}"},
+            indoc::indoc! {"
+            %---10---|%---20---|
+            (foo())#foo{bar = 2,
+                        baz =
+                            {bar,
+                             baz}}"},
         ];
         for text in texts {
             crate::assert_format!(text, Expr);
