@@ -1,4 +1,4 @@
-//! Erlang toplevel components such attributes, directives or declarations.
+//! Erlang top-level components such as attributes, directives or declarations.
 use crate::format::{self, Format};
 use crate::items::atoms::{
     CallbackAtom, DefineAtom, IncludeAtom, IncludeLibAtom, OpaqueAtom, RecordAtom, SpecAtom,
@@ -7,14 +7,13 @@ use crate::items::atoms::{
 use crate::items::expressions::functions::FunctionClause;
 use crate::items::expressions::Expr;
 use crate::items::generics::{
-    Args, Clauses2, Either, Maybe, NonEmptyItems2, Params, Parenthesized2, Tuple,
+    Args, Clauses, Either, Maybe, Params, Parenthesized, TupleLike, WithArrow, WithGuard,
 };
-use crate::items::keywords::{IfKeyword, WhenKeyword};
+use crate::items::keywords::IfKeyword;
 use crate::items::macros::{MacroName, MacroReplacement};
-use crate::items::styles::{RightSpace, Space, TrailingColumns};
 use crate::items::symbols::{
     CloseParenSymbol, ColonSymbol, CommaSymbol, DotSymbol, DoubleColonSymbol, HyphenSymbol,
-    MatchSymbol, OpenParenSymbol, RightArrowSymbol,
+    MatchSymbol, OpenParenSymbol,
 };
 use crate::items::tokens::{AtomToken, StringToken, Token, VariableToken};
 use crate::items::Type;
@@ -43,8 +42,8 @@ pub struct RecordDecl {
     record: RecordAtom,
     open: OpenParenSymbol,
     name: AtomToken,
-    comma: RightSpace<CommaSymbol>,
-    fields: Tuple<RecordField, 3>, // "})."
+    comma: CommaSymbol,
+    fields: TupleLike<RecordField>,
     close: CloseParenSymbol,
     dot: DotSymbol,
 }
@@ -57,17 +56,19 @@ impl Format for RecordDecl {
         fmt.subregion().current_column_as_indent().enter(|fmt| {
             self.name.format(fmt)?;
             self.comma.format(fmt)?;
-            if fmt
-                .subregion()
+            fmt.write_space()?;
+            fmt.subregion()
                 .forbid_multi_line()
                 .forbid_too_long_line()
+                .reset_trailing_columns(2) // ")."
+                .check_trailing_columns(true)
                 .enter(|fmt| self.fields.format(fmt))
-                .is_err()
-            {
-                fmt.write_newline()?;
-                self.fields.format(fmt)?;
-            }
-            Ok(())
+                .or_else(|_| {
+                    fmt.write_newline()?;
+                    fmt.subregion()
+                        .reset_trailing_columns(2) // ")."
+                        .enter(|fmt| self.fields.format(fmt))
+                })
         })?;
         self.close.format(fmt)?;
         self.dot.format(fmt)?;
@@ -75,11 +76,30 @@ impl Format for RecordDecl {
     }
 }
 
-#[derive(Debug, Clone, Span, Parse, Format)]
+#[derive(Debug, Clone, Span, Parse)]
 struct RecordField {
     name: AtomToken,
-    default: Maybe<(Space<MatchSymbol>, Expr)>,
-    r#type: Maybe<(Space<DoubleColonSymbol>, Type)>,
+    default: Maybe<(MatchSymbol, Expr)>,
+    r#type: Maybe<(DoubleColonSymbol, Type)>,
+}
+
+impl Format for RecordField {
+    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
+        self.name.format(fmt)?;
+        if let Some((x, y)) = self.default.get() {
+            fmt.write_space()?;
+            x.format(fmt)?;
+            fmt.write_space()?;
+            y.format(fmt)?;
+        }
+        if let Some((x, y)) = self.r#type.get() {
+            fmt.write_space()?;
+            x.format(fmt)?;
+            fmt.write_space()?;
+            y.format(fmt)?;
+        }
+        Ok(())
+    }
 }
 
 /// `-` (`type` | `opaque`) `$NAME` `(` (`$PARAM` `,`?)* `)` `::` `$TYPE` `.`
@@ -93,16 +113,16 @@ struct RecordField {
 pub struct TypeDecl {
     hyphen: HyphenSymbol,
     kind: Either<TypeAtom, OpaqueAtom>,
-    item: Either<TypeDeclItem, Parenthesized2<TypeDeclItem>>,
+    item: Either<TypeDeclItem, Parenthesized<TypeDeclItem>>,
     dot: DotSymbol,
 }
 
 #[derive(Debug, Clone, Span, Parse)]
 struct TypeDeclItem {
     name: AtomToken,
-    params: TrailingColumns<Params<VariableToken>, 3>, // " ::"
-    delimiter: Space<DoubleColonSymbol>,
-    r#type: TrailingColumns<Type, 1>, // "."
+    params: Params<VariableToken>,
+    delimiter: DoubleColonSymbol,
+    r#type: Type,
 }
 
 impl Format for TypeDeclItem {
@@ -113,19 +133,25 @@ impl Format for TypeDeclItem {
 
         fmt.subregion().current_column_as_indent().enter(|fmt| {
             self.name.format(fmt)?;
-            self.params.format(fmt)?;
+            fmt.subregion()
+                .reset_trailing_columns(3) // " ::"
+                .enter(|fmt| self.params.format(fmt))?;
+            fmt.write_space()?;
             self.delimiter.format(fmt)?;
-            if fmt
-                .subregion()
+            fmt.write_space()?;
+            fmt.subregion()
                 .current_column_as_indent()
                 .forbid_too_long_line()
+                .trailing_columns(1) // "."
+                .check_trailing_columns(true)
                 .enter(|fmt| self.r#type.format(fmt))
-                .is_err()
-            {
-                fmt.subregion()
-                    .indent_offset(2)
-                    .enter_with_newline(|fmt| self.r#type.format(fmt))?;
-            }
+                .or_else(|_| {
+                    fmt.write_newline()?;
+                    fmt.subregion()
+                        .trailing_columns(1) // "."
+                        .indent_offset(2)
+                        .enter(|fmt| self.r#type.format(fmt))
+                })?;
             Ok(())
         })?;
         Ok(())
@@ -139,19 +165,31 @@ impl Format for TypeDeclItem {
 /// - $RETURN: [Type]
 ///
 /// Note that the parenthesized notation like `-spec(foo() -> bar()).` is also acceptable
-#[derive(Debug, Clone, Span, Parse, Format)]
+#[derive(Debug, Clone, Span, Parse)]
 pub struct FunSpec {
     hyphen: HyphenSymbol,
     kind: Either<SpecAtom, CallbackAtom>,
-    item: Either<FunSpecItem, Parenthesized2<FunSpecItem>>,
+    item: Either<FunSpecItem, Parenthesized<FunSpecItem>>,
     dot: DotSymbol,
 }
 
+impl Format for FunSpec {
+    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
+        self.hyphen.format(fmt)?;
+        self.kind.format(fmt)?;
+        fmt.subregion()
+            .reset_trailing_columns(1) // "."
+            .enter(|fmt| self.item.format(fmt))?;
+        self.dot.format(fmt)?;
+        Ok(())
+    }
+}
+
 #[derive(Debug, Clone, Span, Parse)]
 struct FunSpecItem {
     module_name: Maybe<(AtomToken, ColonSymbol)>,
     function_name: AtomToken,
-    clauses: Clauses2<SpecClause, 1>, // trailing: "."
+    clauses: Clauses<SpecClause>,
 }
 
 impl Format for FunSpecItem {
@@ -171,10 +209,8 @@ impl Format for FunSpecItem {
 
 #[derive(Debug, Clone, Span, Parse)]
 struct SpecClause {
-    params: TrailingColumns<Params<Type>, 3>, // trailing: " ->"
-    arrow: Space<RightArrowSymbol>,
-    r#return: Type,
-    constraint: Maybe<Constraint>,
+    params: WithArrow<Params<Type>>,
+    r#return: WithGuard<Type, Type, CommaSymbol>,
 }
 
 impl Format for SpecClause {
@@ -182,64 +218,43 @@ impl Format for SpecClause {
         fmt.subregion()
             .clear_trailing_columns(true)
             .enter(|fmt| self.params.format(fmt))?;
-        self.arrow.format(fmt)?;
 
-        if fmt
-            .subregion()
+        fmt.subregion()
             .forbid_multi_line()
             .forbid_too_long_line()
-            .check_trailing_columns(self.constraint.is_none())
+            .check_trailing_columns(true)
             .enter(|fmt| self.r#return.format(fmt))
-            .is_err()
-        {
-            fmt.write_newline()?;
-            fmt.subregion()
-                .parent_indent()
-                .indent_offset(4)
-                .enter(|fmt| self.r#return.format(fmt))?;
-        }
-
-        if fmt
-            .subregion()
-            .forbid_multi_line()
-            .forbid_too_long_line()
-            .check_trailing_columns(!self.constraint.is_none())
-            .enter(|fmt| self.constraint.format(fmt))
-            .is_err()
-        {
-            fmt.write_newline()?;
-            fmt.subregion()
-                .parent_indent()
-                .indent_offset(8)
-                .enter(|fmt| self.constraint.format(fmt))?;
-        }
-
+            .or_else(|_| {
+                fmt.write_newline()?;
+                fmt.subregion()
+                    .parent_indent()
+                    .indent_offset(4)
+                    .enter(|fmt| self.r#return.format(fmt))
+            })?;
         Ok(())
     }
 }
 
-#[derive(Debug, Clone, Span, Parse, Format)]
-struct Constraint {
-    when: Space<WhenKeyword>,
-    constraints: NonEmptyItems2<Type, CommaSymbol>,
-}
-
 /// (`$NAME` `(` (`$PARAM` `,`?)* `)` (`when` `$GUARD`)? `->` `$BODY` `;`?)+ `.`
 ///
 /// - $NAME: [AtomToken]
 /// - $PARAM: [Expr]
 /// - $GUARD: ([Expr] (`,` | `;`)?)+
 /// - $BODY: ([Expr] `,`?)+
-#[derive(Debug, Clone, Span, Parse, Format)]
+#[derive(Debug, Clone, Span, Parse)]
 pub struct FunDecl {
-    clauses: Clauses2<FunDeclClause, 1>, // trailing: "."
+    clauses: Clauses<FunctionClause<AtomToken>>,
     dot: DotSymbol,
 }
 
-#[derive(Debug, Clone, Span, Parse, Format)]
-struct FunDeclClause {
-    name: AtomToken,
-    clause: FunctionClause,
+impl Format for FunDecl {
+    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
+        fmt.subregion()
+            .reset_trailing_columns(1) // "."
+            .enter(|fmt| self.clauses.format(fmt))?;
+        self.dot.format(fmt)?;
+        Ok(())
+    }
 }
 
 /// `-` `$NAME` `$ARGS`? `.`
@@ -247,14 +262,28 @@ struct FunDeclClause {
 /// - $NAME: [AtomToken] | `if`
 /// - $ARGS: `(` (`$ARG` `,`?)* `)`
 /// - $ARG: [Expr]
-#[derive(Debug, Clone, Span, Parse, Format)]
+#[derive(Debug, Clone, Span, Parse)]
 pub struct Attr {
     hyphen: HyphenSymbol,
     name: Either<AtomToken, IfKeyword>,
-    items: Maybe<TrailingColumns<Args<Expr>, 1>>, // "."
+    items: Maybe<Args<Expr>>,
     dot: DotSymbol,
 }
 
+impl Format for Attr {
+    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
+        self.hyphen.format(fmt)?;
+        self.name.format(fmt)?;
+        if let Some(items) = self.items.get() {
+            fmt.subregion()
+                .reset_trailing_columns(1) // "."
+                .enter(|fmt| items.format(fmt))?;
+        }
+        self.dot.format(fmt)?;
+        Ok(())
+    }
+}
+
 /// `-` `define` `(` `$NAME` `$VARS`? `,` `REPLACEMENT`* `)` `.`
 ///
 /// - $NAME: [AtomToken] | [VariableToken]
@@ -267,8 +296,8 @@ pub struct DefineDirective {
     open: OpenParenSymbol,
     macro_name: MacroName,
     variables: Maybe<Params<VariableToken>>,
-    comma: RightSpace<CommaSymbol>,
-    replacement: TrailingColumns<MacroReplacement, 2>, // ")."
+    comma: CommaSymbol,
+    replacement: MacroReplacement,
     close: CloseParenSymbol,
     dot: DotSymbol,
 }
@@ -283,7 +312,7 @@ impl DefineDirective {
     }
 
     pub fn replacement(&self) -> &[Token] {
-        self.replacement.get().tokens()
+        self.replacement.tokens()
     }
 }
 
@@ -296,17 +325,19 @@ impl Format for DefineDirective {
             self.macro_name.format(fmt)?;
             self.variables.format(fmt)?;
             self.comma.format(fmt)?;
-            if fmt
-                .subregion()
+            fmt.write_space()?;
+            fmt.subregion()
                 .forbid_multi_line()
                 .forbid_too_long_line()
+                .reset_trailing_columns(2) // ")."
+                .check_trailing_columns(true)
                 .enter(|fmt| self.replacement.format(fmt))
-                .is_err()
-            {
-                fmt.write_newline()?;
-                self.replacement.format(fmt)?;
-            }
-            Ok(())
+                .or_else(|_| {
+                    fmt.write_newline()?;
+                    fmt.subregion()
+                        .reset_trailing_columns(2) // ")."
+                        .enter(|fmt| self.replacement.format(fmt))
+                })
         })?;
         self.close.format(fmt)?;
         self.dot.format(fmt)?;
@@ -529,13 +560,13 @@ mod tests {
                      (XXX,
                       YYY) -> Z."},
             indoc::indoc! {"
-            -spec id(X) -> X
-                          when X :: tuple()."},
+            -spec id(X) ->
+                      X when X :: tuple()."},
             indoc::indoc! {"
-            -spec id(X) -> X
-                          when is_subtype(X,
-                                          atom()),
-                               X :: atom()."},
+            -spec id(X) ->
+                      X when is_subtype(X,
+                                        atom()),
+                             X :: atom()."},
             indoc::indoc! {"
             -callback foobar(atom()) ->
                           {atom(),
@@ -545,7 +576,7 @@ mod tests {
             -spec foobar(A) ->
                       {atom(),
                        atom()}
-                          when A :: atom();
+                        when A :: atom();
                         (a) ->
                       b."},
             indoc::indoc! {"
