@@ -1,7 +1,8 @@
 use crate::format::{self, Format, Formatter};
-use crate::items::styles::{Newline, TrailingColumns};
+use crate::items::keywords::WhenKeyword;
 use crate::items::symbols::{
-    CloseBraceSymbol, CloseParenSymbol, CommaSymbol, OpenBraceSymbol, OpenParenSymbol,
+    CloseBraceSymbol, CloseParenSymbol, CloseSquareSymbol, CommaSymbol, DoubleLeftAngleSymbol,
+    DoubleRightAngleSymbol, OpenBraceSymbol, OpenParenSymbol, OpenSquareSymbol, RightArrowSymbol,
     SemicolonSymbol,
 };
 use crate::items::tokens::WhitespaceToken;
@@ -9,23 +10,47 @@ use crate::parse::{self, Parse, ResumeParse, TokenStream};
 use crate::span::{Position, Span};
 
 #[derive(Debug, Clone)]
-pub struct Maybe<T>(Either<T, WhitespaceToken>);
+pub struct Null(WhitespaceToken);
 
-impl<T> Maybe<T> {
-    pub fn is_none(&self) -> bool {
-        matches!(self.0, Either::B(_))
+impl Span for Null {
+    fn start_position(&self) -> Position {
+        self.0.end_position() // TODO: note
     }
 
+    fn end_position(&self) -> Position {
+        self.0.start_position() // TODO: note
+    }
+}
+
+impl Parse for Null {
+    fn parse(ts: &mut parse::TokenStream) -> parse::Result<Self> {
+        ts.current_whitespace_token().map(Self)
+    }
+}
+
+impl Format for Null {
+    fn format(&self, _: &mut format::Formatter) -> format::Result<()> {
+        Ok(())
+    }
+}
+
+#[derive(Debug, Clone, Span, Parse)]
+pub struct Maybe<T>(Either<T, Null>);
+
+impl<T> Maybe<T> {
     pub fn from_item(item: T) -> Self {
         Self(Either::A(item))
     }
 
     pub fn from_position(position: Position) -> Self {
-        Self(Either::B(WhitespaceToken::new(position, position)))
+        Self(Either::B(Null(WhitespaceToken::new(position, position))))
     }
 
     pub fn none(ts: &mut TokenStream) -> parse::Result<Self> {
-        ts.current_whitespace_token().map(Either::B).map(Self)
+        ts.current_whitespace_token()
+            .map(Null)
+            .map(Either::B)
+            .map(Self)
     }
 
     pub fn get(&self) -> Option<&T> {
@@ -37,31 +62,6 @@ impl<T> Maybe<T> {
     }
 }
 
-impl<T: Span> Span for Maybe<T> {
-    fn start_position(&self) -> Position {
-        match &self.0 {
-            Either::A(x) => x.start_position(),
-            Either::B(x) => x.end_position(), // TODO: add note comment
-        }
-    }
-
-    fn end_position(&self) -> Position {
-        match &self.0 {
-            Either::A(x) => x.end_position(),
-            Either::B(x) => x.start_position(), // TODO: add note comment
-        }
-    }
-}
-
-impl<T: Parse> Parse for Maybe<T> {
-    fn parse(ts: &mut TokenStream) -> parse::Result<Self> {
-        ts.parse()
-            .map(Either::A)
-            .or_else(|_| ts.current_whitespace_token().map(Either::B))
-            .map(Self)
-    }
-}
-
 impl<T: Format> Format for Maybe<T> {
     fn format(&self, fmt: &mut Formatter) -> format::Result<()> {
         if let Some(x) = self.get() {
@@ -99,31 +99,7 @@ impl<T: Format> Format for Parenthesized<T> {
         self.open.format(fmt)?;
         fmt.subregion()
             .current_column_as_indent()
-            .trailing_columns(1) // ')'
-            .enter(|fmt| self.item.format(fmt))?;
-        self.close.format(fmt)?;
-        Ok(())
-    }
-}
-
-#[derive(Debug, Clone, Span, Parse)]
-pub struct Parenthesized2<T> {
-    open: OpenParenSymbol,
-    item: TrailingColumns<T, 1>, // trailing: ")"
-    close: CloseParenSymbol,
-}
-
-impl<T> Parenthesized2<T> {
-    pub fn get(&self) -> &T {
-        self.item.get()
-    }
-}
-
-impl<T: Format> Format for Parenthesized2<T> {
-    fn format(&self, fmt: &mut Formatter) -> format::Result<()> {
-        self.open.format(fmt)?;
-        fmt.subregion()
-            .current_column_as_indent()
+            .trailing_columns(1) // ")"
             .enter(|fmt| self.item.format(fmt))?;
         self.close.format(fmt)?;
         Ok(())
@@ -131,40 +107,40 @@ impl<T: Format> Format for Parenthesized2<T> {
 }
 
 #[derive(Debug, Clone, Span, Parse, Format)]
-pub struct Params<T>(Parenthesized2<Items2<T>>);
+pub struct Params<T>(Parenthesized<Items<T>>);
 
 impl<T> Params<T> {
     pub fn get(&self) -> &[T] {
-        self.0.get().get()
+        self.0.get().items()
     }
 }
 
 #[derive(Debug, Clone, Span, Parse, Format)]
 pub struct Args<T>(Parenthesized<Items<T>>);
 
-#[derive(Debug, Clone, Span, Parse, Format)]
-pub struct Args2<T>(Parenthesized2<Items2<T>>);
-
-impl<T> Args2<T> {
+impl<T> Args<T> {
     pub fn get(&self) -> &[T] {
-        self.0.get().get()
+        self.0.get().items()
     }
 }
 
 #[derive(Debug, Clone)]
 pub struct NonEmptyItems<T, D = CommaSymbol> {
-    // TODO: private
-    pub items: Vec<T>,
-    pub delimiters: Vec<D>,
+    items: Vec<T>,
+    delimiters: Vec<D>,
 }
 
 impl<T, D> NonEmptyItems<T, D> {
-    pub fn get(&self) -> &[T] {
+    pub fn items(&self) -> &[T] {
         &self.items
     }
+
+    pub fn delimiters(&self) -> &[D] {
+        &self.delimiters
+    }
 }
 
-impl<T: Span, D: Span> Span for NonEmptyItems<T, D> {
+impl<T: Span, D> Span for NonEmptyItems<T, D> {
     fn start_position(&self) -> Position {
         self.items[0].start_position()
     }
@@ -189,7 +165,12 @@ impl<T: Parse, D: Parse> Parse for NonEmptyItems<T, D> {
 impl<T: Format, D: Format> NonEmptyItems<T, D> {
     fn format_items(&self, fmt: &mut Formatter, multi_line: bool) -> format::Result<()> {
         for (item, delimiter) in self.items.iter().zip(self.delimiters.iter()) {
-            item.format(fmt)?;
+            let trailing_columns = fmt
+                .item_formatted_text(delimiter)
+                .map_or(1, |s| s.trim_end().len());
+            fmt.subregion()
+                .reset_trailing_columns(trailing_columns)
+                .enter(|fmt| item.format(fmt))?;
             delimiter.format(fmt)?;
             if multi_line {
                 fmt.write_newline()?;
@@ -200,20 +181,24 @@ impl<T: Format, D: Format> NonEmptyItems<T, D> {
         self.items.last().expect("unreachable").format(fmt)?;
         Ok(())
     }
+
+    pub fn format_multiline(&self, fmt: &mut Formatter) -> format::Result<()> {
+        fmt.subregion()
+            .current_column_as_indent()
+            .enter(|fmt| self.format_items(fmt, true))
+    }
 }
 
 impl<T: Format, D: Format> Format for NonEmptyItems<T, D> {
     fn format(&self, fmt: &mut Formatter) -> format::Result<()> {
-        if fmt
-            .subregion()
-            .forbid_multi_line()
-            .forbid_too_long_line()
-            .enter(|fmt| self.format_items(fmt, false))
-            .is_err()
-        {
-            self.format_items(fmt, true)?;
-        }
-        Ok(())
+        fmt.subregion().current_column_as_indent().enter(|fmt| {
+            fmt.subregion()
+                .forbid_multi_line()
+                .forbid_too_long_line()
+                .check_trailing_columns(true)
+                .enter(|fmt| self.format_items(fmt, false))
+                .or_else(|_| self.format_items(fmt, true))
+        })
     }
 }
 
@@ -221,9 +206,17 @@ impl<T: Format, D: Format> Format for NonEmptyItems<T, D> {
 pub struct Items<T, D = CommaSymbol>(Maybe<NonEmptyItems<T, D>>);
 
 impl<T, D> Items<T, D> {
-    pub fn get(&self) -> &[T] {
+    pub fn items(&self) -> &[T] {
         if let Some(x) = self.0.get() {
-            x.get()
+            x.items()
+        } else {
+            &[]
+        }
+    }
+
+    pub fn delimiters(&self) -> &[D] {
+        if let Some(x) = self.0.get() {
+            x.delimiters()
         } else {
             &[]
         }
@@ -231,256 +224,164 @@ impl<T, D> Items<T, D> {
 }
 
 #[derive(Debug, Clone, Span, Parse)]
-pub struct Elements<T>(Items<T>);
-
-impl<T: Format> Elements<T> {
-    fn format_packed_items(&self, fmt: &mut Formatter) -> format::Result<()> {
-        let (items, delimiters) =
-            if let Some((items, delimiters)) = self.0 .0.get().map(|x| (&x.items, &x.delimiters)) {
-                (items, delimiters)
-            } else {
-                return Ok(());
-            };
-
-        let max_columns = fmt.region_config().max_columns;
-        fn is_head(fmt: &Formatter) -> bool {
-            fmt.current_column() == fmt.region_config().indent
-        }
-
-        for (item, delimiter) in items.iter().zip(delimiters.iter()) {
-            if !is_head(fmt) && fmt.current_column() + item.len() + delimiter.len() > max_columns {
-                fmt.write_newline()?;
-            }
+pub struct MaybePackedItems<T, D = CommaSymbol>(Items<T, D>);
 
+impl<T: Format, D: Format> MaybePackedItems<T, D> {
+    fn format_packed_item(
+        &self,
+        fmt: &mut Formatter,
+        item: &T,
+        delimiter: Option<&D>,
+    ) -> format::Result<()> {
+        let format = |fmt: &mut Formatter| {
             item.format(fmt)?;
-            delimiter.format(fmt)?;
-            fmt.write_space()?;
+            if let Some(d) = delimiter {
+                d.format(fmt)?;
+                fmt.write_space()?;
+            }
+            Ok(())
+        };
+
+        let is_head = fmt.current_column() == fmt.region_config().indent;
+        if is_head {
+            format(fmt)
+        } else {
+            fmt.subregion()
+                .forbid_multi_line()
+                .forbid_too_long_line()
+                .check_trailing_columns(delimiter.is_none())
+                .enter(format)
+                .or_else(|_| {
+                    fmt.write_newline()?;
+                    format(fmt)
+                })
+        }
+    }
+
+    fn format_packed_items(&self, fmt: &mut Formatter) -> format::Result<()> {
+        for (item, delimiter) in self.0.items().iter().zip(self.0.delimiters().iter()) {
+            self.format_packed_item(fmt, item, Some(delimiter))?;
         }
 
-        let item = items.last().expect("unreachable");
-        if !is_head(fmt) && fmt.current_column() + item.len() > max_columns {
-            fmt.write_newline()?;
-        }
-        item.format(fmt)?;
+        let item = self.0.items().last().expect("unreachable");
+        self.format_packed_item(fmt, item, None)?;
 
         Ok(())
     }
 }
 
-impl<T: Format> Format for Elements<T> {
+impl<T: Format, D: Format> Format for MaybePackedItems<T, D> {
     fn format(&self, fmt: &mut Formatter) -> format::Result<()> {
-        fmt.subregion()
-            .current_column_as_indent()
-            .trailing_columns(2) // '>>' or ']' or ... (use const generic?)
-            .enter(|fmt| {
-                let packed = self.0.get().iter().all(|x| x.should_be_packed());
-                if packed {
-                    self.format_packed_items(fmt)
-                } else {
-                    self.0.format(fmt)
-                }
-            })
+        if self.0.items().is_empty() {
+            Ok(())
+        } else if self.0.items().iter().all(|x| x.should_be_packed()) {
+            fmt.subregion()
+                .current_column_as_indent()
+                .enter(|fmt| self.format_packed_items(fmt))
+        } else {
+            self.0.format(fmt)
+        }
     }
 }
 
 #[derive(Debug, Clone, Span, Parse)]
-pub struct Clauses<T>(NonEmptyItems<T, Newline<SemicolonSymbol>>);
+pub struct ListLike<T, D = CommaSymbol> {
+    open: OpenSquareSymbol,
+    items: MaybePackedItems<T, D>,
+    close: CloseSquareSymbol,
+}
 
-impl<T: Format> Format for Clauses<T> {
+impl<T: Format, D: Format> Format for ListLike<T, D> {
     fn format(&self, fmt: &mut Formatter) -> format::Result<()> {
+        self.open.format(fmt)?;
         fmt.subregion()
-            .current_column_as_indent()
-            .enter(|fmt| self.0.format(fmt))
-    }
-}
-
-#[derive(Debug, Clone)]
-pub struct MaybeRepeat<T>(Vec<T>);
-
-impl<T> MaybeRepeat<T> {
-    pub fn get(&self) -> &[T] {
-        &self.0
-    }
-}
-
-impl<T: Span> Span for MaybeRepeat<T> {
-    fn start_position(&self) -> Position {
-        self.0[0].start_position()
-    }
-
-    fn end_position(&self) -> Position {
-        self.0[self.0.len() - 1].end_position()
-    }
-}
-
-impl<T: Parse> Parse for MaybeRepeat<T> {
-    fn parse(ts: &mut TokenStream) -> parse::Result<Self> {
-        let mut items = vec![ts.parse()?];
-        while let Ok(item) = ts.parse() {
-            items.push(item);
-        }
-        Ok(Self(items))
-    }
-}
-
-impl<T: Format> Format for MaybeRepeat<T> {
-    fn format(&self, fmt: &mut Formatter) -> format::Result<()> {
-        for item in &self.0 {
-            item.format(fmt)?;
-        }
+            .trailing_columns(1) // "]"
+            .enter(|fmt| self.items.format(fmt))?;
+        self.close.format(fmt)?;
         Ok(())
     }
 }
 
-// TODO: TupleLike
-#[derive(Debug, Clone, Span, Parse, Format)]
-pub struct Tuple<T, const N: usize> {
+#[derive(Debug, Clone, Span, Parse)]
+pub struct TupleLike<T> {
     open: OpenBraceSymbol,
-    items: TupleElements<T, N>,
+    items: MaybePackedItems<T>,
     close: CloseBraceSymbol,
 }
 
-#[derive(Debug, Clone, Span, Parse)]
-pub struct TupleElements<T, const N: usize>(Items<T>);
-
-impl<T: Format, const N: usize> TupleElements<T, N> {
-    fn format_packed_items(&self, fmt: &mut Formatter) -> format::Result<()> {
-        let (items, delimiters) =
-            if let Some((items, delimiters)) = self.0 .0.get().map(|x| (&x.items, &x.delimiters)) {
-                (items, delimiters)
-            } else {
-                return Ok(());
-            };
-
-        let max_columns = fmt.region_config().max_columns;
-        fn is_head(fmt: &Formatter) -> bool {
-            fmt.current_column() == fmt.region_config().indent
-        }
-
-        for (item, delimiter) in items.iter().zip(delimiters.iter()) {
-            if !is_head(fmt) && fmt.current_column() + item.len() + delimiter.len() > max_columns {
-                fmt.write_newline()?;
-            }
-
-            item.format(fmt)?;
-            delimiter.format(fmt)?;
-            fmt.write_space()?;
-        }
-
-        let item = items.last().expect("unreachable");
-        if !is_head(fmt) && fmt.current_column() + item.len() > max_columns {
-            fmt.write_newline()?;
-        }
-        item.format(fmt)?;
-
-        Ok(())
-    }
-}
-
-impl<T: Format, const N: usize> Format for TupleElements<T, N> {
+impl<T: Format> Format for TupleLike<T> {
     fn format(&self, fmt: &mut Formatter) -> format::Result<()> {
+        self.open.format(fmt)?;
         fmt.subregion()
-            .current_column_as_indent()
-            .trailing_columns(N) // TODO: remove (may always be 1?)
-            .enter(|fmt| {
-                let packed = self.0.get().iter().all(|x| x.should_be_packed());
-                if packed {
-                    self.format_packed_items(fmt)
-                } else {
-                    self.0.format(fmt)
-                }
-            })
-    }
-}
-
-// TODO: refactor
-#[derive(Debug, Clone, Span, Parse)]
-pub struct NonEmptyItems2<T, D>(NonEmptyItems<T, D>);
-
-impl<T, D> NonEmptyItems2<T, D> {
-    pub fn get(&self) -> &[T] {
-        self.0.get()
-    }
-}
-
-impl<T: Format, D: Format> NonEmptyItems2<T, D> {
-    fn format_items(&self, fmt: &mut Formatter, multi_line: bool) -> format::Result<()> {
-        for (item, delimiter) in self.0.items.iter().zip(self.0.delimiters.iter()) {
-            fmt.subregion()
-                .reset_trailing_columns(delimiter.len())
-                .enter(|fmt| item.format(fmt))?;
-            delimiter.format(fmt)?;
-            if multi_line {
-                fmt.write_newline()?;
-            } else {
-                fmt.write_space()?;
-            }
-        }
-        fmt.subregion()
-            .check_trailing_columns(true)
-            .enter(|fmt| self.0.items.last().expect("unreachable").format(fmt))?;
+            .trailing_columns(1) // "}"
+            .enter(|fmt| self.items.format(fmt))?;
+        self.close.format(fmt)?;
         Ok(())
     }
 }
 
-impl<T: Format, D: Format> Format for NonEmptyItems2<T, D> {
+#[derive(Debug, Clone, Span, Parse)]
+pub struct BitstringLike<T> {
+    open: DoubleLeftAngleSymbol,
+    items: MaybePackedItems<T>,
+    close: DoubleRightAngleSymbol,
+}
+
+impl<T: Format> Format for BitstringLike<T> {
+    fn format(&self, fmt: &mut Formatter) -> format::Result<()> {
+        self.open.format(fmt)?;
+        fmt.subregion()
+            .trailing_columns(2) // ">>"
+            .enter(|fmt| self.items.format(fmt))?;
+        self.close.format(fmt)?;
+        Ok(())
+    }
+}
+
+#[derive(Debug, Clone, Span, Parse)]
+pub struct Clauses<T>(NonEmptyItems<T, SemicolonSymbol>);
+
+impl<T> Clauses<T> {
+    pub fn items(&self) -> &[T] {
+        self.0.items()
+    }
+}
+
+impl<T: Format> Format for Clauses<T> {
     fn format(&self, fmt: &mut Formatter) -> format::Result<()> {
         fmt.subregion().current_column_as_indent().enter(|fmt| {
-            if fmt
-                .subregion()
-                .forbid_multi_line()
-                .forbid_too_long_line()
-                .enter(|fmt| self.format_items(fmt, false))
-                .is_err()
-            {
-                self.format_items(fmt, true)?;
+            for (item, delimiter) in self.0.items().iter().zip(self.0.delimiters().iter()) {
+                fmt.subregion()
+                    .reset_trailing_columns(1) // ";"
+                    .enter(|fmt| item.format(fmt))?;
+                delimiter.format(fmt)?;
+                fmt.write_newline()?;
             }
+            self.0.items().last().expect("unreachable").format(fmt)?;
             Ok(())
         })
     }
 }
 
-#[derive(Debug, Clone, Span, Parse)]
-pub struct Clauses2<T, const N: usize>(NonEmptyItems2<T, Newline<SemicolonSymbol>>);
-
-impl<T: Format, const N: usize> Format for Clauses2<T, N> {
-    fn format(&self, fmt: &mut Formatter) -> format::Result<()> {
-        fmt.subregion()
-            .current_column_as_indent()
-            .trailing_columns2(N)
-            .enter(|fmt| self.0.format(fmt))
-    }
-}
-
-#[derive(Debug, Clone, Span, Parse, Format)]
-pub struct Items2<T, D = CommaSymbol>(Maybe<NonEmptyItems2<T, D>>);
-
-impl<T, D> Items2<T, D> {
-    pub fn get(&self) -> &[T] {
-        if let Some(x) = self.0.get() {
-            x.get()
-        } else {
-            &[]
-        }
-    }
-}
-
-pub trait NeedsBeforeSpace {
-    fn needs_before_space(&self, fmt: &format::Formatter) -> bool;
-}
-
 #[derive(Debug, Clone, Span, Parse)]
 pub struct UnaryOpLike<O, T> {
     op: O,
     item: T,
 }
 
-impl<O: Format + NeedsBeforeSpace, T: Format> Format for UnaryOpLike<O, T> {
+impl<O: Format, T: Format> Format for UnaryOpLike<O, T> {
     fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
-        if self.op.needs_before_space(fmt) {
+        if fmt
+            .item_formatted_text(&self.op)
+            .map_or(false, |s| s.starts_with(fmt.last_char()))
+        {
             fmt.write_space()?;
         }
         self.op.format(fmt)?;
+        if fmt.last_char().is_alphabetic() {
+            fmt.write_space()?;
+        }
+
         self.item.format(fmt)?;
         Ok(())
     }
@@ -490,18 +391,23 @@ impl<O: Format + NeedsBeforeSpace, T: Format> Format for UnaryOpLike<O, T> {
     }
 }
 
-// TODO: rename
-#[derive(Debug, Clone, Span, Parse)]
-pub struct BinaryOpLike<T0, O, T1, const I: usize> {
-    left: T0,
-    op: O,
-    right: T1,
+pub trait BinaryOpStyle {
+    fn indent_offset(&self) -> usize;
+
+    fn allow_newline(&self) -> bool;
+
+    fn should_pack(&self) -> bool;
 }
 
-impl<T0: Parse, O: Parse, T1: Parse, const I: usize> ResumeParse<T0>
-    for BinaryOpLike<T0, O, T1, I>
-{
-    fn resume_parse(ts: &mut parse::TokenStream, left: T0) -> parse::Result<Self> {
+#[derive(Debug, Clone, Span, Parse)]
+pub struct BinaryOpLike<L, O, R> {
+    pub left: L,
+    pub op: O,
+    pub right: R,
+}
+
+impl<L: Parse, O: Parse, R: Parse> ResumeParse<L> for BinaryOpLike<L, O, R> {
+    fn resume_parse(ts: &mut parse::TokenStream, left: L) -> parse::Result<Self> {
         Ok(Self {
             left,
             op: ts.parse()?,
@@ -510,31 +416,113 @@ impl<T0: Parse, O: Parse, T1: Parse, const I: usize> ResumeParse<T0>
     }
 }
 
-impl<T0: Format, O: Format, T1: Format, const I: usize> Format for BinaryOpLike<T0, O, T1, I> {
+impl<L: Format, O: Format + BinaryOpStyle, R: Format> Format for BinaryOpLike<L, O, R> {
     fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
-        fmt.subregion().current_column_as_indent().enter(|fmt| {
-            fmt.subregion()
-                .reset_trailing_columns(self.op.len())
-                .enter(|fmt| self.left.format(fmt))?;
-            self.op.format(fmt)?;
+        let trailing_columns = fmt
+            .item_formatted_text(&self.op)
+            .map_or(1, |s| s.trim_end().len())
+            + 1;
 
-            if fmt.current_relative_column() <= I {
-                // Inserting a newline cannot shorten the line length.
-                self.right.format(fmt)?
-            } else if fmt
-                .subregion()
-                .forbid_too_long_line()
-                .forbid_multi_line()
-                .check_trailing_columns(true)
-                .enter(|fmt| self.right.format(fmt))
-                .is_err()
-            {
-                fmt.subregion().indent_offset(I).enter(|fmt| {
-                    fmt.write_newline()?;
-                    self.right.format(fmt)
-                })?;
-            }
+        fmt.subregion()
+            .reset_trailing_columns(trailing_columns)
+            .enter(|fmt| self.left.format(fmt))?;
+        fmt.write_space()?;
+        self.op.format(fmt)?;
+        fmt.write_space()?;
+
+        let indent_offset = self.op.indent_offset();
+        if !self.op.allow_newline() || fmt.current_relative_column() <= indent_offset {
+            // Inserting a newline is forbiddend or cannot shorten the line length.
+            return self.right.format(fmt);
+        }
+
+        let mut options = fmt
+            .subregion()
+            .forbid_too_long_line()
+            .check_trailing_columns(true);
+        if !self.op.should_pack() {
+            options = options.forbid_multi_line();
+        }
+        if options.enter(|fmt| self.right.format(fmt)).is_err() {
+            fmt.subregion().indent_offset(indent_offset).enter(|fmt| {
+                fmt.write_newline()?;
+                self.right.format(fmt)
+            })?;
+        }
+        Ok(())
+    }
+}
+
+#[derive(Debug, Clone, Span, Parse)]
+pub struct WithArrow<T> {
+    item: T,
+    arrow: RightArrowSymbol,
+}
+
+impl<T: Format> Format for WithArrow<T> {
+    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
+        fmt.subregion()
+            .reset_trailing_columns(3) // " ->"
+            .enter(|fmt| {
+                self.item.format(fmt)?;
+                fmt.write_space()?;
+                self.arrow.format(fmt)?;
+                fmt.write_space()?;
+                Ok(())
+            })
+        // TODO: .enter_with_trailer(|fmt| {...}, |fmt| {})
+    }
+}
+
+#[derive(Debug, Clone, Span, Parse)]
+pub struct WithGuard<T, U, D = Either<CommaSymbol, SemicolonSymbol>> {
+    item: T,
+    guard: Maybe<Guard<U, D>>,
+}
+
+impl<T: Format, U: Format, D: Format> Format for WithGuard<T, U, D> {
+    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
+        if let Some(guard) = self.guard.get() {
+            fmt.subregion()
+                .clear_trailing_columns(true)
+                .enter(|fmt| self.item.format(fmt))?;
+            fmt.write_space()?;
+            guard.format(fmt)?;
+        } else {
+            self.item.format(fmt)?;
+        }
+        Ok(())
+    }
+}
+
+#[derive(Debug, Clone, Span, Parse)]
+struct Guard<T, D> {
+    when: WhenKeyword,
+    conditions: NonEmptyItems<T, D>,
+}
+
+impl<T: Format, D: Format> Format for Guard<T, D> {
+    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
+        let format = |fmt: &mut format::Formatter| {
+            self.when.format(fmt)?;
+            fmt.write_space()?;
+            self.conditions.format(fmt)?;
             Ok(())
-        })
+        };
+
+        if fmt.current_relative_column() <= 2 {
+            format(fmt)?;
+        } else if fmt
+            .subregion()
+            .forbid_too_long_line()
+            .forbid_multi_line()
+            .check_trailing_columns(true)
+            .enter(format)
+            .is_err()
+        {
+            fmt.write_newline()?;
+            fmt.subregion().indent_offset(2).enter(format)?;
+        }
+        Ok(())
     }
 }
