@@ -3,19 +3,16 @@
 //! <https://www.erlang.org/doc/reference_manual/typespec.html>
 use crate::format::{self, Format};
 use crate::items::generics::{
-    Args2, BinaryOpLike, Either, Maybe, NeedsBeforeSpace, NonEmptyItems2, Params, Parenthesized,
-    Parenthesized2, Tuple, UnaryOpLike,
+    Args, BinaryOpLike, BinaryOpStyle, BitstringLike, Either, ListLike, Maybe, NonEmptyItems,
+    Params, Parenthesized, TupleLike, UnaryOpLike,
 };
 use crate::items::keywords::{
     BandKeyword, BnotKeyword, BorKeyword, BslKeyword, BsrKeyword, BxorKeyword, DivKeyword,
     FunKeyword, RemKeyword,
 };
-use crate::items::styles::{RightSpace, Space, TrailingColumns};
 use crate::items::symbols::{
-    CloseSquareSymbol, ColonSymbol, CommaSymbol, DoubleColonSymbol, DoubleDotSymbol,
-    DoubleLeftAngleSymbol, DoubleRightAngleSymbol, DoubleRightArrowSymbol, HyphenSymbol,
-    MapMatchSymbol, MultiplySymbol, OpenSquareSymbol, PlusSymbol, RightArrowSymbol, SharpSymbol,
-    TripleDotSymbol, VerticalBarSymbol,
+    ColonSymbol, DoubleColonSymbol, DoubleDotSymbol, HyphenSymbol, MultiplySymbol, PlusSymbol,
+    RightArrowSymbol, SharpSymbol, TripleDotSymbol, VerticalBarSymbol,
 };
 use crate::items::tokens::{AtomToken, CharToken, IntegerToken, VariableToken};
 use crate::items::Type;
@@ -41,7 +38,18 @@ impl Parse for NonUnionType {
 
 /// [Type] `|` [Type]
 #[derive(Debug, Clone, Span, Parse, Format)]
-pub struct UnionType(NonEmptyItems2<NonUnionType, Space<VerticalBarSymbol>>);
+pub struct UnionType(NonEmptyItems<NonUnionType, UnionDelimiter>);
+
+#[derive(Debug, Clone, Span, Parse)]
+struct UnionDelimiter(VerticalBarSymbol);
+
+impl Format for UnionDelimiter {
+    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
+        fmt.write_space()?;
+        self.0.format(fmt)?;
+        Ok(())
+    }
+}
 
 #[derive(Debug, Clone, Span, Parse, Format)]
 enum NonLeftRecursiveType {
@@ -59,18 +67,29 @@ enum NonLeftRecursiveType {
 }
 
 /// [VariableToken] `::` [Type]
-#[derive(Debug, Clone, Span, Parse, Format)]
+#[derive(Debug, Clone, Span, Parse)]
 pub struct AnnotatedVariableType {
     variable: VariableToken,
-    colon: Space<DoubleColonSymbol>,
+    colon: DoubleColonSymbol,
     ty: Type,
 }
 
+impl Format for AnnotatedVariableType {
+    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
+        self.variable.format(fmt)?;
+        fmt.write_space()?;
+        self.colon.format(fmt)?;
+        fmt.write_space()?;
+        self.ty.format(fmt)?;
+        Ok(())
+    }
+}
+
 /// [Type] `$OP` [Type]
 ///
 /// - $OP: [BinaryOp]
 #[derive(Debug, Clone, Span, Parse, Format)]
-pub struct BinaryOpType(BinaryOpLike<NonLeftRecursiveType, BinaryOp, Type, 0>);
+pub struct BinaryOpType(BinaryOpLike<NonLeftRecursiveType, BinaryOp, Type>);
 
 impl ResumeParse<NonLeftRecursiveType> for BinaryOpType {
     fn resume_parse(
@@ -84,19 +103,33 @@ impl ResumeParse<NonLeftRecursiveType> for BinaryOpType {
 /// `*` | `+` | `-` | `div` | `rem` | `band` | `bor` | `bxor` | `bsl` | `bsr` | `..`
 #[derive(Debug, Clone, Span, Parse, Format)]
 pub enum BinaryOp {
-    Mul(Space<MultiplySymbol>),
-    Plus(Space<PlusSymbol>),
-    Minus(Space<HyphenSymbol>),
-    Div(Space<DivKeyword>),
-    Rem(Space<RemKeyword>),
-    Band(Space<BandKeyword>),
-    Bor(Space<BorKeyword>),
-    Bxor(Space<BxorKeyword>),
-    Bsl(Space<BslKeyword>),
-    Bsr(Space<BsrKeyword>),
+    Mul(MultiplySymbol),
+    Plus(PlusSymbol),
+    Minus(HyphenSymbol),
+    Div(DivKeyword),
+    Rem(RemKeyword),
+    Band(BandKeyword),
+    Bor(BorKeyword),
+    Bxor(BxorKeyword),
+    Bsl(BslKeyword),
+    Bsr(BsrKeyword),
     Range(DoubleDotSymbol),
 }
 
+impl BinaryOpStyle for BinaryOp {
+    fn indent_offset(&self) -> usize {
+        0
+    }
+
+    fn allow_newline(&self) -> bool {
+        true
+    }
+
+    fn should_pack(&self) -> bool {
+        true
+    }
+}
+
 /// `$OP` [Type]
 ///
 /// - $OP: [UnaryOp]
@@ -108,33 +141,50 @@ pub struct UnaryOpType(UnaryOpLike<UnaryOp, NonLeftRecursiveType>);
 pub enum UnaryOp {
     Plus(PlusSymbol),
     Minus(HyphenSymbol),
-    Bnot(Space<BnotKeyword>),
-}
-
-impl NeedsBeforeSpace for UnaryOp {
-    fn needs_before_space(&self, fmt: &format::Formatter) -> bool {
-        matches!(
-            (fmt.last_char(), self),
-            ('-', UnaryOp::Minus(_)) | ('+', UnaryOp::Plus(_))
-        )
-    }
+    Bnot(BnotKeyword),
 }
 
 /// `fun` `(` (`$PARAMS` `->` `$RETURN`)? `)`
 ///
 /// - $PARAMS: `(` `...` `)` | `(` ([Type] `,`?)* `)`
 /// - $RETURN: [Type]
-#[derive(Debug, Clone, Span, Parse, Format)]
+#[derive(Debug, Clone, Span, Parse)]
 pub struct FunctionType {
-    fun: Space<FunKeyword>,
-    params_and_return: Parenthesized2<Maybe<FunctionParamsAndReturn>>,
+    fun: FunKeyword,
+    params_and_return: Parenthesized<Maybe<FunctionParamsAndReturn>>,
 }
 
-type FunctionParamsAndReturn = BinaryOpLike<FunctionParams, Space<RightArrowSymbol>, Type, 8>;
+impl Format for FunctionType {
+    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
+        self.fun.format(fmt)?;
+        fmt.write_space()?;
+        self.params_and_return.format(fmt)?;
+        Ok(())
+    }
+}
+
+type FunctionParamsAndReturn = BinaryOpLike<FunctionParams, RightArrowDelimiter, Type>;
+
+#[derive(Debug, Clone, Span, Parse, Format)]
+struct RightArrowDelimiter(RightArrowSymbol);
+
+impl BinaryOpStyle for RightArrowDelimiter {
+    fn indent_offset(&self) -> usize {
+        8
+    }
+
+    fn allow_newline(&self) -> bool {
+        true
+    }
+
+    fn should_pack(&self) -> bool {
+        false
+    }
+}
 
 #[derive(Debug, Clone, Span, Parse, Format)]
 enum FunctionParams {
-    Any(Parenthesized2<TripleDotSymbol>),
+    Any(Parenthesized<TripleDotSymbol>),
     Params(Params<Type>),
 }
 
@@ -156,75 +206,27 @@ pub enum LiteralType {
 pub struct MfargsType {
     module: Maybe<(AtomToken, ColonSymbol)>,
     name: AtomToken,
-    args: Args2<Type>,
+    args: Args<Type>,
 }
 
-type ListItem = (Type, Maybe<(RightSpace<CommaSymbol>, TripleDotSymbol)>);
-
-/// `[]` | `[` [Type] (`,` `...`)? `]`
-#[derive(Debug, Clone, Span, Parse)]
-pub struct ListType {
-    open: OpenSquareSymbol,
-    item: Maybe<ListItem>,
-    close: CloseSquareSymbol,
-}
-
-impl ListType {
-    fn format_item(
-        &self,
-        fmt: &mut format::Formatter,
-        ty: &Type,
-        rest: Option<&(RightSpace<CommaSymbol>, TripleDotSymbol)>,
-    ) -> format::Result<()> {
-        if let Some((comma, triple_dot)) = rest {
-            fmt.subregion()
-                .reset_trailing_columns(1)
-                .enter(|fmt| ty.format(fmt))?;
-            fmt.subregion()
-                .forbid_too_long_line()
-                .check_trailing_columns(true)
-                .enter(|fmt| {
-                    comma.format(fmt)?;
-                    triple_dot.format(fmt)
-                })
-                .or_else(|_| {
-                    comma.format(fmt)?;
-                    fmt.write_newline()?;
-                    triple_dot.format(fmt)
-                })?;
-        } else {
-            ty.format(fmt)?;
-        }
-        Ok(())
-    }
-}
-
-impl Format for ListType {
-    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
-        self.open.format(fmt)?;
-        if let Some((ty, rest)) = self.item.get() {
-            fmt.subregion()
-                .current_column_as_indent()
-                .trailing_columns2(1) // "," or "]"
-                .enter(|fmt| self.format_item(fmt, ty, rest.get()))?;
-        }
-        self.close.format(fmt)?;
-        Ok(())
-    }
-}
+/// `[` (`$ITEM` `,`?)* `]`
+///
+/// - $ITEM: [Type] | `...`
+#[derive(Debug, Clone, Span, Parse, Format)]
+pub struct ListType(ListLike<Either<Type, TripleDotSymbol>>);
 
 /// `{` ([Type] `,`)* `}`
 #[derive(Debug, Clone, Span, Parse, Format)]
-pub struct TupleType(Tuple<Type, 1>);
+pub struct TupleType(TupleLike<Type>);
 
 /// `#` `{` ([Type] (`:=` | `=>`) [Type] `,`?)* `}`
 #[derive(Debug, Clone, Span, Parse, Format)]
 pub struct MapType {
     sharp: SharpSymbol,
-    items: Tuple<MapItem, 1>,
+    items: TupleLike<MapItem>,
 }
 
-type MapItem = BinaryOpLike<Type, Space<Either<DoubleRightArrowSymbol, MapMatchSymbol>>, Type, 4>;
+type MapItem = BinaryOpLike<Type, crate::items::expressions::maps::MapDelimiter, Type>;
 
 /// `#` `$NAME` `{` (`$FIELD` `,`?)* `}`
 ///
@@ -234,30 +236,34 @@ type MapItem = BinaryOpLike<Type, Space<Either<DoubleRightArrowSymbol, MapMatchS
 pub struct RecordType {
     sharp: SharpSymbol,
     name: AtomToken,
-    fields: Tuple<RecordItem, 1>,
+    fields: TupleLike<RecordItem>,
 }
 
-type RecordItem = BinaryOpLike<AtomToken, Space<DoubleColonSymbol>, Type, 4>;
+type RecordItem = BinaryOpLike<AtomToken, DoubleColonDelimiter, Type>;
+
+#[derive(Debug, Clone, Span, Parse, Format)]
+struct DoubleColonDelimiter(DoubleColonSymbol);
+
+impl BinaryOpStyle for DoubleColonDelimiter {
+    fn indent_offset(&self) -> usize {
+        4
+    }
+
+    fn allow_newline(&self) -> bool {
+        true
+    }
+
+    fn should_pack(&self) -> bool {
+        false
+    }
+}
 
 /// `<<` `$BITS_SIZE`? `,`? `$UNIT_SIZE`? `>>`
 ///
 /// - $BITS_SIZE: `_` `:` [Type]
 /// - $UNIT_SIZE: `_` `:` `_` `*` [Type]
 #[derive(Debug, Clone, Span, Parse, Format)]
-pub struct BitstringType {
-    open: DoubleLeftAngleSymbol,
-    size: Maybe<TrailingColumns<BitstringSize, 2>>, // trailing: ">>"
-    close: DoubleRightAngleSymbol,
-}
-
-#[derive(Debug, Clone, Span, Parse, Format)]
-enum BitstringSize {
-    BitsAndUnit(
-        Box<BinaryOpLike<BitstringBitsSize, RightSpace<CommaSymbol>, BitstringUnitSize, 0>>,
-    ),
-    Unit(Box<BitstringUnitSize>),
-    Bits(Box<BitstringBitsSize>),
-}
+pub struct BitstringType(BitstringLike<Either<BitstringUnitSize, BitstringBitsSize>>);
 
 #[derive(Debug, Clone, Span, Parse, Format)]
 struct BitstringBitsSize {
@@ -412,7 +418,7 @@ mod tests {
             %---10---|%---20---|
             foo |
             (3 + 10) |
-            -1..+20"},
+            -1 .. +20"},
         ];
         for text in texts {
             crate::assert_format!(text, Type);
