@@ -1,7 +1,9 @@
 % From: http://www1.erlang.org/examples/examples-2.0.html
 connect(Host, User, Password) ->
-    {ftp_server, Host} !
-        {connect, self(), User, Password},
+    {ftp_server, Host} ! {connect,
+                          self(),
+                          User,
+                          Password},
     receive
         {ftp_server, Reply} ->
             Reply;
@@ -44,8 +46,7 @@ lls() ->
 put(Handle, File) ->
     case file:read_file(File) of
         {ok, Contents} ->
-            remote(Handle,
-                   {put, File, Contents});
+            remote(Handle, {put, File, Contents});
         Other ->
             Other
     end.
