@@ -25,35 +25,32 @@ loop(Users, N) ->
         {connect, Pid, User, Password} ->
             io:format("connection request from:~p ~p ~p~n",
                       [Pid, User, Password]),
-            case member({User, Password}, Users) of
+            case member({User, Password},
+                        Users) of
                 true ->
                     Max = max_connections(),
                     if
                         N > Max ->
-                            Pid !
-                                {ftp_server,
-                                 {error,
-                                  too_many_connections}},
+                            Pid ! {ftp_server,
+                                   {error,
+                                    too_many_connections}},
                             loop(Users, N);
                         true ->
                             New =
                                 spawn_link(?MODULE,
                                            handler,
                                            [Pid]),
-                            Pid !
-                                {ftp_server,
-                                 {ok, New}},
+                            Pid ! {ftp_server,
+                                   {ok, New}},
                             loop(Users, N + 1)
                     end;
                 false ->
-                    Pid !
-                        {ftp_server,
-                         {error, rejected}},
+                    Pid ! {ftp_server,
+                           {error, rejected}},
                     loop(Users, N)
             end;
         {'EXIT', Pid} ->
-            io:format("Handler ~p died~n",
-                      [Pid]),
+            io:format("Handler ~p died~n", [Pid]),
             loop(Users, lists:max(N - 1, 0));
         Any ->
             io:format("received:~p~n", [Any]),
